import { useState, useEffect, useMemo, useContext } from "react";
import { CanvasContext } from "./CanvasContext.js";
import { compareObjects } from "./utils.js";
const routePattern = /^(\/:?[a-zA-Z0-9_]+)+$/;
function getRouteURL(host, route, params) {
    if (!routePattern.test(route)) {
        throw new Error("Invalid route");
    }
    const queryParams = { ...params };
    const path = route.slice(1).split("/");
    const pathComponents = path.map((component) => {
        if (component.startsWith(":")) {
            const value = params[component.slice(1)];
            if (value === undefined) {
                throw new Error(`missing parameter ${component}`);
            }
            else if (typeof value !== "string") {
                throw new Error(`URL parameter ${component} must be a string`);
            }
            delete queryParams[component.slice(1)];
            return encodeURIComponent(value);
        }
        else {
            return component;
        }
    });
    if (host.endsWith("/")) {
        host = host.slice(0, -1);
    }
    const queryComponents = Object.entries(queryParams).map(([key, value]) => `${key}=${encodeURIComponent(JSON.stringify(value))}`);
    if (queryComponents.length > 0) {
        return `${host}/${pathComponents.join("/")}?${queryComponents.join("&")}`;
    }
    else {
        return `${host}/${pathComponents.join("/")}`;
    }
}
export function useRoute(route, params, options = { subscribe: true }, callback) {
    const { host, ws, data: applicationData } = useContext(CanvasContext);
    if (host === null) {
        throw new Error("No API endpoint provided! you must provide an API endpoint in a parent Canvas element.");
    }
    const [isLoading, setIsLoading] = useState(true);
    const [error, setError] = useState(null);
    const [data, setData] = useState(null);
    const url = useMemo(() => getRouteURL(host, route, params), [host, route, params]);
    const readyState = ws?.readyState;
    const subscribe = options.subscribe ?? true;
    const listener = useMemo(() => (evt) => {
        if (evt.data.toString() === "pong")
            return;
        try {
            const message = JSON.parse(evt.data.toString());
            if (message.route === route && compareObjects(message.params, params)) {
                setData(message.data);
                setIsLoading(false);
                if (callback)
                    setTimeout(() => callback(message.data, null));
            }
        }
        catch (err) {
            console.log("ws: failed to parse message", evt.data);
        }
    }, [callback]);
    useEffect(() => {
        if (applicationData === null || ws === null) {
            return;
        }
        else if (!applicationData.routes.includes(route)) {
            setError(new Error(`${applicationData.uri} has no route ${JSON.stringify(route)}`));
            setData(null);
            setIsLoading(false);
            if (callback)
                setTimeout(() => callback(null, error));
            return;
        }
        setIsLoading(true);
        if (subscribe) {
            if (ws.readyState === ws.OPEN) {
                ws.addEventListener("message", listener);
                ws.send(JSON.stringify({ action: "subscribe", data: { route, params } }));
                return () => {
                    ws.removeEventListener("message", listener);
                    if (ws.readyState === ws.OPEN) {
                        ws.send(JSON.stringify({ action: "unsubscribe", data: { route, params } }));
                    }
                };
            }
        }
        else {
            fetch(url)
                .then((res) => res.json())
                .then((data) => {
                setError(null);
                setData(data);
                setIsLoading(false);
                if (callback)
                    setTimeout(() => callback(data, null));
            })
                .catch((err) => {
                setError(err);
                setData(null);
                setIsLoading(false);
                if (callback)
                    setTimeout(() => callback(null, err));
            });
        }
    }, [route, url, !!applicationData, subscribe, ws, readyState, callback]);
    return { error, data, isLoading };
}
