import { useCallback, useContext, useEffect, useState } from "react";
import { CanvasContext } from "./CanvasContext.js";
import { getCanvasSessionKey, getRecentBlock, urlJoin } from "./utils.js";
export function useSession(signer) {
    const { host, data, setSigner, actionSigner, setActionSigner, sessionExpiration, setSessionExpiration } = useContext(CanvasContext);
    const [error, setError] = useState(null);
    const [isLoading, setIsLoading] = useState(true);
    const [isPending, setIsPending] = useState(false);
    const [signerAddress, setSignerAddress] = useState(null);
    useEffect(() => {
        if (signer === null) {
            setSignerAddress(null);
            setSigner(null);
        }
        else {
            signer.getAddress().then((address) => {
                setSignerAddress(address);
                setSigner(signer);
            });
        }
        setActionSigner(null);
        setSessionExpiration(null);
    }, [signer]);
    useEffect(() => {
        if (host === null || data === null || signerAddress === null) {
            return;
        }
        setIsLoading(false);
        const sessionKey = getCanvasSessionKey(signerAddress);
        const item = localStorage.getItem(sessionKey);
        if (item === null) {
            return;
        }
        let sessionObject;
        try {
            sessionObject = JSON.parse(item);
        }
        catch (err) {
            localStorage.removeItem(sessionKey);
            return;
        }
        if (!isSessionObject(sessionObject)) {
            localStorage.removeItem(sessionKey);
            return;
        }
        const { app, sessionPrivateKey, expiration } = sessionObject;
        if (data.uri !== app || expiration < Date.now()) {
            localStorage.removeItem(sessionKey);
            return;
        }
        signer.createActionSigner(sessionPrivateKey).then((actionSigner) => {
            setActionSigner(actionSigner);
        });
        setSessionExpiration(expiration);
    }, [host, data, signerAddress]);
    const login = useCallback(async () => {
        if (host === null) {
            return setError(new Error("no host configured"));
        }
        else if (signer === null) {
            return setError(new Error("login() called without a signer"));
        }
        else if (signerAddress === null) {
            return setError(new Error("login() called before the signer was ready"));
        }
        else if (data === null) {
            return setError(new Error("login() called before the application connection was established"));
        }
        setIsPending(true);
        try {
            const timestamp = Date.now();
            const sessionDuration = 86400 * 1000;
            const actionSigner = await signer.createActionSigner();
            const sessionObject = {
                app: data.uri,
                sessionPrivateKey: actionSigner.privateKey,
                expiration: timestamp + sessionDuration,
            };
            const chain = await signer.getChain();
            const chainId = await signer.getChainId();
            let block;
            try {
                block = await getRecentBlock(host, chain, chainId);
            }
            catch (err) {
                block = await signer.getRecentBlock();
            }
            const payload = {
                from: signerAddress,
                app: data.uri,
                sessionAddress: actionSigner.address,
                sessionDuration,
                sessionIssued: timestamp,
                block: block.blockhash,
                chain: block.chain,
                chainId: block.chainId,
            };
            const session = await signer.signSessionPayload(payload);
            const res = await fetch(urlJoin(host, "sessions"), {
                method: "POST",
                headers: { "Content-Type": "application/json" },
                body: JSON.stringify(session),
            });
            if (!res.ok) {
                const message = await res.text();
                throw new Error(message);
            }
            const sessionKey = getCanvasSessionKey(signerAddress);
            localStorage.setItem(sessionKey, JSON.stringify(sessionObject));
            setActionSigner(actionSigner);
            setSessionExpiration(sessionObject.expiration);
            setError(null);
        }
        catch (err) {
            console.error(err);
            if (err instanceof Error) {
                setError(err);
            }
            else {
                throw err;
            }
        }
        finally {
            setIsPending(false);
        }
    }, [host, data, signer, signerAddress]);
    const logout = useCallback(() => {
        setActionSigner(null);
        setSessionExpiration(null);
        if (signerAddress !== null) {
            const sessionKey = getCanvasSessionKey(signerAddress);
            localStorage.removeItem(sessionKey);
        }
    }, [signerAddress]);
    const sessionAddress = actionSigner && actionSigner.address;
    return {
        error,
        isLoading,
        isPending,
        sessionAddress,
        sessionExpiration,
        login,
        logout,
    };
}
function isSessionObject(obj) {
    return (typeof obj === "object" &&
        typeof obj.app === "string" &&
        typeof obj.sessionPrivateKey === "string" &&
        typeof obj.expiration === "number");
}
